/*****************************************************************************/
/* Full Bridge Inverter on 28035                                             */
/* Company: Transphorm Inc.                                                  */
/* Date: Mar 2020                                                            */
/* Location: Goleta, CA                                                      */
/* Coder: Z.W.                                                               */
/* Version 1.0  Initialization                                               */
/* Fsw = 50,66,100,133kHz                                                    */
/* Line frequency setting: 50Hz or 60Hz                                      */
/* For TO220,TO247kW Demo Inverter Board                                     */
/* Open loop, with current or voltage sampling                               */
/* Add OCP function, turn off active device but keep freewheeling device on  */
/* for lower conduction loss.                                                */
/* 50,66, 100, 133kHz fsw, 50 or 60 Hz line frequency                        */
/*****************************************************************************/

#include "DSP2803x_Device.h"
#include "DSP2803x_Examples.h"
#include "IQmathLib.h"
#include "math.h"
#include "DSP2803x_Adc.h"

/******************************************************************************/
/* Customer settings Begin                                                    */
/* FSS_SETTING: 50, 66 OR 100 for 50kHz, 66kHz or 100kHz switching frequency  */
/* For 50kHz or 66kHz, choose 500uH inductor                                  */
/*                                                                            */
/* FLINE_SETTING: Set 50 or 60 for 50Hz or 60Hz Line frequency                */
/*                                                                            */
/* MOD_INDEX: Modulation index 0~0.95                                         */
/*                                                                            */
/* OCP_LV_UP, 100 for 1A, 2000 for 20A                                        */
/******************************************************************************/

// Set switching frequency, //50 or 100 ,66,133kHz
#define FSW_SETTING  50

// Set line frequency, 50, or 60 Hz
#define   FLINE_SETTING       60   //60 or 50

// 0-1. PWM Modulation index
#define MOD_INDEX 0.98 //0.92

// OCP setting, 1 for 0.01A, OCP<2500 for 50mOhm, OCP<3400 for 35mOhm
#define OCP_LV_UP 5000  //1000 for 10A peak current

// OCP Time out, using RMS current to count ocp time, RMS_OCP = OCP_PK*0.71 = OCP_PK*23/32
#define OCP_TIME_OUT  60      //60 Line cycle, i.e. 1 second
/*Customer Settings End                                                      */

#define OCP_ACT  0   //0 recommended default for only turn off active device, but keep freewheeling device on
                     //1 for turn off all the devices, resulting in higher conduction loss

#define PI 3.1415926        //3.14159265 * 2^11
#define TBPRD_145KHz 207    //Counter for 145KHz; 60M/145K/2 =
#define TBPRD_133KHz 220    //Counter for 136.5KHz; 60M/133K/2 =
#define TBPRD_100KHz 300    //Counter for 100KHz
#define TBPRD_50KHz  600    //Counter for 50KHz
#define TBPRD_66KHz  454    //Counter for 66KHz
#define TBPRD_10KHz  3000   // Counter for 10kHz

#pragma CODE_SECTION(epwm1_isr, "ramfuncs");
#pragma CODE_SECTION(epwm3_isr, "ramfuncs");

// Prototype statements for functions found within this file.
void InitEPwm1Gpio(void);
void InitEPwm2Gpio(void);
void InitEPwm3Gpio(void);
void InitEPwm1Example(void);
void InitEPwm2Example(void);
void InitEPwm3Example(void);

void ConfigAdc1(void);  //config ADC at the beginning to sample Vref
void ConfigAdc2(void);  //config ADC to sample current and voltage sequentially for control purpose
void InitGPIOs(void);
int GetPWM_Mode(void);

interrupt void epwm1_isr(void);
interrupt void epwm3_isr(void);

//These are defined by the linker
extern Uint16 RamfuncsLoadStart;
extern Uint16 RamfuncsLoadEnd;
extern Uint16 RamfuncsRunStart;

// Global variables
Uint16 Sum1;
Uint16 Sum2;
Uint16 Sum3;
Uint16 Ref1V65;
Uint16 Ifb_V_ref;
Uint16 Ifb_U_ref;

int Vac;
int Vdc;
int Ifb_V;
int Ifb_U;
int Vac_real;
int Vdc_real;
int Ifb_V_real;
int Ifb_U_real;
Uint16 Ifb_real_max1;
Uint16 Ifb_real_max2;
Uint16 Ifb_real_max;
int MainPoint;
long VSqrtSum;
long ISqrtSum_U;
long ISqrtSum_V;
int Vac_rms;
int Vac_rms_f;
int Ifb_V_rms;
int Ifb_U_rms;
int Ifb_V_rms_f;
int Ifb_U_rms_f;
int If_rms_OCP;
Uint16 RMS_OCP_Cnt;
Uint16 RMS_OCP_Flag;
Uint16 flag_ZC;
Uint16 SubCnt;
long tempV1;
long tempI1;
long tempI2;

Uint16  N_fsw;                	//the N th switching cycle in one line cycle;
Uint16	TBPRD_Setting;		  	// Time Based Period Register setting
Uint16  SWperLineCyc_Setting; 	// How many numbers switching cycle for one line cycle; depends on 50KHz or 100KHz
_iq   mod_u_iq;                //The modulation index of phase U in IQ format
_iq   IQCMPRA;                // COMPARE A in IQ Format
_iq   theta_iq;                 // voltage angle in IQ Format
_iq   sintheta_iq;             //sin(theta) in IQ Format
_iq   IQCMPRB;                // COMPARE B in IQ Format
Uint16  fLine;       			//flag for line waveform

Uint16	Cnt_SWperLineCyc_145KHz;//Counts number switching cycle in one line cycle;
Uint16	Cnt_SWperLineCyc_133KHz;//Counts number switching cycle in one line cycle;
Uint16	Cnt_SWperLineCyc_100KHz;//Counts number switching cycle in one line cycle;
Uint16	Cnt_SWperLineCyc_50KHz; //Counts number switching cycle in one line cycle;
Uint16	Cnt_SWperLineCyc_75KHz; //Counts number switching cycle in one line cycle;
Uint16	Cnt_SWperLineCyc_66KHz; //Counts number switching cycle in one line cycle;

//parameters to calculate theta
Uint16  Cnt_A;
Uint16  Shift_B;
Uint16  Shift_C;

Uint16  Int1Count;
Uint16  EPwm3TimerIntCount;


int main(void)
{
		// Initialize System Control:
		// PLL, WatchDog, enable Peripheral Clocks
		// This function function is found in the DSP2803x_SysCtrl.c file.
	    InitSysCtrl();

		// Initalize GPIO:
		// init GPIO pins for ePWM1, ePWM2, ePWM3
		// These functions are at the end of this file
		InitEPwm1Gpio();
		InitEPwm2Gpio();
		InitEPwm3Gpio();

		InitGPIOs();

		// Clear all interrupts and initialize PIE vector table:
		// Disable CPU interrupts
		DINT;

		// Initialize the PIE control registers to their default state.
		// The default state is all PIE interrupts disabled and flags
		// are cleared.
		// This function is found in the DSP2803x_PieCtrl.c file.
		InitPieCtrl();

		// Disable CPU interrupts and clear all CPU interrupt flags:
		IER = 0x0000;
		IFR = 0x0000;

		// Initialize the PIE vector table with pointers to the shell Interrupt
		// Service Routines (ISR).
		// This will populate the entire table, even if the interrupt
		// is not used
		// The shell ISR routines are found in DSP2803x_DefaultIsr.c.
		// This function is found in DSP2803x_PieVect.c.
		InitPieVectTable();

		// Interrupts that are used in this example are re-mapped to
		// ISR functions found within this file.
		EALLOW;
		PieVectTable.EPWM1_INT = &epwm1_isr;

		PieVectTable.EPWM3_INT = &epwm3_isr;
		EDIS;

		// Copy time critical code and Flash setup code to RAM
		// The  RamfuncsLoadStart, RamfuncsLoadEnd, and RamfuncsRunStart
		// symbols are created by the linker. Refer to the F28035.cmd file.
		MemCopy(&RamfuncsLoadStart, &RamfuncsLoadEnd, &RamfuncsRunStart);

		// Call Flash Initialization to setup flash wait states
		// This function must reside in RAM
		InitFlash();
	    InitAdc();

		// initialize variables
		mod_u_iq = _IQ(MOD_INDEX); //Modulation index 0.94 max in IQ Format


		// Initialize variables
		Sum1 = 0;
        Sum2 = 0;
        Sum3 = 0;
		Ref1V65 = 2048;
		Ifb_V_ref = 2048;
		Ifb_U_ref = 2048;

		Vac = 0;
		Vdc = 0;
		Ifb_V = 0;
		Ifb_U = 0;
		Vac_real = 0;
		Vdc_real = 0;
		Ifb_V_real = 0;
		Ifb_U_real = 0;
        Ifb_real_max1 = 0;
        Ifb_real_max2 = 0;
        Ifb_real_max = 0;
		MainPoint = 0;
		VSqrtSum = 0;
		ISqrtSum_U = 0;
		ISqrtSum_V = 0;
		Vac_rms = 0;
		Vac_rms_f = 0;
		Ifb_V_rms = 0;
		Ifb_U_rms = 0;
		Ifb_V_rms_f = 0;
		Ifb_U_rms_f = 0;
		If_rms_OCP = OCP_LV_UP;
		If_rms_OCP = (int)(((long)If_rms_OCP *23)>>5);
		RMS_OCP_Cnt = 0;
		RMS_OCP_Flag = 0;
		flag_ZC = 0;
		SubCnt = 0;
		tempV1 = 0;
        tempI1 = 0;
        tempI2 = 0;
        Cnt_A = 2;
        Shift_B = 2;
        Shift_C = 11;

		// initialize PWM and frequency
		N_fsw = 0;                	//the N th switching cycle in one line cycle;

		TBPRD_Setting = 300;		  	// Time Based Period Register setting, 100kHz default
	    fLine = 0;       			//flag for line waveform
	    flag_ZC = 0;

        #if FLINE_SETTING == 60
            Cnt_SWperLineCyc_145KHz = 2416;     //Counts number switching cycle in one line cycle, 145k/60 =2217;
            //2416= 1/(217/(256*2048)) A=217, B= 8(256), C = 11(2048)
            Cnt_SWperLineCyc_133KHz = 2275;     //Counts number switching cycle in one line cycle, 133k/60 =2217;2275
            //2274 = 1/(461/(512*2048)) A=461, B= 9(512), C = 11(2048)
            Cnt_SWperLineCyc_100KHz = 1664;//Counts number switching cycle in one line cycle;
            //1664= 1/(315/(256*2048)) A=315, B= 8(256), C = 11(2048)
            Cnt_SWperLineCyc_50KHz = 832; //Counts number switching cycle in one line cycle;
            //832 = 1/(315/(128*2048)) A=315, B= 7(256), C = 11(2048)
            Cnt_SWperLineCyc_66KHz = 1100; //Counts number switching cycle in one line cycle;
            //1100 = 1/(119/(64*2048)) A=119, B= 6(64), C = 11(2048)
        #elif FLINE_SETTING == 50
            Cnt_SWperLineCyc_145KHz = 2900;     //Counts number switching cycle in one line cycle, 145k/50 =2900;
            //2900= 1/(723/(1024*2048)) A=723, B= 10(1024), C = 11(2048)
            Cnt_SWperLineCyc_133KHz = 2730;     //136.5k/50 =2730;
            //2730= 1/(3/(4*2048)) A=3, B= 2(4), C = 11(2048)
            Cnt_SWperLineCyc_100KHz = 2048;// 2048 for 102.4kHz
            //2048 = 1/(2/(2*2048)),A=2, B=1, C=11
            Cnt_SWperLineCyc_50KHz = 1024; // 1024 for 51.2kHz
            //1024 = 1/(2/(2*1024)),A=2, B=1, C=10
            Cnt_SWperLineCyc_66KHz = 1324; // 1332 = 66k/50 = 1320
            //1322 = 1/(99/(2*1024)),A=99, B=7, C=10
         #endif

		// Operate at 99.72KHz
        #if FSW_SETTING == 133
			TBPRD_Setting = TBPRD_133KHz;
			SWperLineCyc_Setting = Cnt_SWperLineCyc_133KHz;  		//
            Cnt_A = 461;
            Shift_B = 9;
            Shift_C = 11;
            #if FLINE_SETTING == 50
                Cnt_A = 3;
                Shift_B = 2;
                Shift_C = 11;
            #endif
        #elif FSW_SETTING == 100

			TBPRD_Setting = TBPRD_100KHz; //300
			SWperLineCyc_Setting = Cnt_SWperLineCyc_100KHz;  		//1666
	        Cnt_A = 315;
	        Shift_B = 8;
	        Shift_C = 11;

            #if FLINE_SETTING == 50
			    Cnt_A = 2;
			    Shift_B = 1;
			    Shift_C = 11;
            #endif
        #elif FSW_SETTING == 66

			TBPRD_Setting = TBPRD_66KHz;
			SWperLineCyc_Setting = Cnt_SWperLineCyc_66KHz;  		//
            Cnt_A = 119;
            Shift_B = 6;
            Shift_C = 11;
            #if FLINE_SETTING == 50
                Cnt_A = 99;
                Shift_B = 7;
                Shift_C = 10;
            #endif

        #elif FSW_SETTING == 145

			TBPRD_Setting = TBPRD_145KHz;
			SWperLineCyc_Setting = Cnt_SWperLineCyc_145KHz;  		//
            Cnt_A = 217;
            Shift_B = 8;
            Shift_C = 11;
            #if FLINE_SETTING == 50
                Cnt_A = 723;
                Shift_B = 10;
                Shift_C = 11;
            #endif

        #else  //default 50kHz

			TBPRD_Setting = TBPRD_50KHz;
			SWperLineCyc_Setting = Cnt_SWperLineCyc_50KHz;  		//
            Cnt_A = 315;
            Shift_B = 7;
            Shift_C = 11;
            #if FLINE_SETTING == 50
                Cnt_A = 2;
                Shift_B = 1;
                Shift_C = 10;
            #endif

        #endif

	    ConfigAdc1();  //configure ADC to sample Vref get the Vref value

	    EALLOW;
	    // Enabled ADCINT1
	    AdcRegs.INTSEL1N2.bit.INT1E = 1;
	    // Enable continuous sampling for ADCINT1 and ADCINT2
	    AdcRegs.INTSEL1N2.bit.INT1CONT = 0;
	    //ADCINTs trigger at end of conversion
	    AdcRegs.ADCCTL1.bit.INTPULSEPOS = 1;
	    // Setup ADCINT1 and ADCINT2 trigger source
	    AdcRegs.INTSEL1N2.bit.INT1SEL = 8;      //EOC1 triggers ADCINT1
	    EDIS;

	    AdcRegs.ADCSOCFRC1.all = 0x1fff;  // force SOC flag 0-12 to be 1 to start ADC conversion

	    while (AdcRegs.ADCINTFLG.bit.ADCINT1 == 0)
	    {

	    }  // wait until 13 conversions are over
	    AdcRegs.ADCINTFLGCLR.bit.ADCINT1 = 1;   //Must clear ADCINT1 flag since INT1CONT = 0

	    // calculate the midpoint of Vac
	    Sum1 += AdcResult.ADCRESULT1;  //discard ADCRESULT0, from errata the first one is not reliable
	    Sum1 += AdcResult.ADCRESULT2;
        Sum1 += AdcResult.ADCRESULT3;
        Sum1 += AdcResult.ADCRESULT4;

        // calculate the midpoint of Iac_V
        Sum2 += AdcResult.ADCRESULT5;
        Sum2 += AdcResult.ADCRESULT6;
        Sum2 += AdcResult.ADCRESULT7;
        Sum2 += AdcResult.ADCRESULT8;

        // calculate the midpoint of Iac_U
        Sum3 += AdcResult.ADCRESULT9;
        Sum3 += AdcResult.ADCRESULT10;
        Sum3 += AdcResult.ADCRESULT11;
        Sum3 += AdcResult.ADCRESULT12;

	    Ref1V65 = Sum1 >> 2;  //get reference 1.65V
	    Ifb_V_ref = Sum2 >> 2; //get reference of Ifb_V channel
	    Ifb_U_ref = Sum3 >> 2; // get reference of Ifb_U channel

		EALLOW;
		SysCtrlRegs.PCLKCR0.bit.TBCLKSYNC = 0;
		EDIS;

		InitEPwm1Example();
		InitEPwm2Example();
		InitEPwm3Example();

	    ConfigAdc2(); //configure ADC for control purpose, sample current and voltage

		EALLOW;
		SysCtrlRegs.PCLKCR0.bit.TBCLKSYNC = 1;
		EDIS;


    Int1Count = 0;
    EPwm3TimerIntCount = 0;
	asm(" RPT #100 || NOP");       		// must wait at least 30 cycles for the BSY bit to show up?? This is a must, although I do not know why?????
	asm(" RPT #100 || NOP");
	asm(" RPT #100 || NOP");

	// Enable interrupts
	// Enable CPU INT3 which is connected to EPWM1(100KHz) and EPWM3(10KHz) INT:
	IER |= M_INT3;

	// Enable EPWM INTn in the PIE: Group 3 interrupt 1-3
	PieCtrlRegs.PIEIER3.bit.INTx1 = 1;
	PieCtrlRegs.PIEIER3.bit.INTx3 = 1;

	// Enable global Interrupts and higher priority real-time debug events:
	EINT;   // Enable Global interrupt INTM
	ERTM;   // Enable Global realtime interrupt DBGM
		for(;;)
		{
			asm("  NOP");
			asm("  NOP");
		}
}


interrupt void epwm1_isr(void)
{
    if (N_fsw < SWperLineCyc_Setting )
    {
        N_fsw ++;
    }
    else
    {
        N_fsw = 0;
        flag_ZC = 1; //zerocrossing
        SubCnt = 1; //start to calculate RMS
    }

    //to improve current limit performance
    if(N_fsw == (SWperLineCyc_Setting - 1))  //in the beginning of positive half cycle
    {
        EPwm1Regs.ETSEL.bit.INTSEL = ET_CTR_ZERO;        // INT on counter-zero
        EPwm1Regs.ETPS.bit.INTPRD = ET_1ST;             // Generate pulse on 1st event

        EPwm1Regs.ETSEL.bit.SOCASEL = ET_CTR_ZERO;       // Select SOC on counter-zero event
        EPwm1Regs.ETPS.bit.SOCAPRD  = ET_1ST;           // Generate pulse on 1st event
    }
    if(N_fsw == ((SWperLineCyc_Setting >> 1) - 1)) //in the beginning of negative half cycle
    {
        EPwm1Regs.ETSEL.bit.INTSEL = ET_CTR_PRD;        // INT on counter-period
        EPwm1Regs.ETPS.bit.INTPRD = ET_1ST;             // Generate pulse on 1st event

        EPwm1Regs.ETSEL.bit.SOCASEL = ET_CTR_PRD;       // Select SOC on counter-period event
        EPwm1Regs.ETPS.bit.SOCAPRD  = ET_1ST;           // Generate pulse on 1st event
    }


    //N_fsw is also for calculating RMS value at any mode;

   //set SOC0 channel select to ADCINA0 ---- Vac
   //set SOC1 channel select to ADCINA0 ---- Vac
   //set SOC2 channel select to ADCINA3 ---- Vdc
   //set SOC3 channel select to ADCINA4 ---- Ifb_V
   //set SOC4 channel select to ADCINB4 ---- Ifb_U
   // read all ADC registers,Vac, Vdc,Ifb_V,Ifb_U
    // Kvdc = 5.506, Vdc 4096/5.506 = 743V
    // Kvac = 4.563, Vac 2048/4.563=317Vac,448Vdc
    // Kifb = 34.158 Ifb = 2048/34.158 = 60A
    Vac = AdcResult.ADCRESULT0;  //dummy read, first conversion not reliable (according to errata)
    Vac = AdcResult.ADCRESULT1;
    Vdc = AdcResult.ADCRESULT2;
    Ifb_V = AdcResult.ADCRESULT3;
    Ifb_U = AdcResult.ADCRESULT4;

    Vac = Vac - Ref1V65;
    Vac_real = (int)(((long)Vac * 561) >> 8); //4.563*561/256 =10, 1 for 0.1V
    //Vdc_real = (int)(((long)Vdc * 1683) >> 10) - 30; //6.084, 5.506*465/256=10, 1 for 0.1V
    Vdc_real = (int)(((long)Vdc * 465) >> 8) - 65; //6.084, 5.506*465/256=10, 1 for 0.1V
    Ifb_V = (int)Ifb_V_ref - 28 - Ifb_V;//-30
    Ifb_U = Ifb_U - Ifb_U_ref + 28;//+30
    Ifb_V_real = (int)(((long)Ifb_V * 1499) >> 9); //34.158*1499/512 =10, 1 for 0.01A
    Ifb_U_real = (int)(((long)Ifb_U * 1499) >> 9); //34.158*1499/512 =10, 1 for 0.01A

    if(abs(Ifb_V_real) > Ifb_real_max)
    {
        Ifb_real_max = abs(Ifb_V_real);
    }
    if(abs(Ifb_U_real) > Ifb_real_max)
    {
        Ifb_real_max = abs(Ifb_U_real);
    }

    //|IV| or |IU| > 3A
    if(RMS_OCP_Flag == 1)
    {
        EALLOW;
        EPwm1Regs.TZFRC.bit.OST = 1;
        EPwm2Regs.TZFRC.bit.OST = 1;    //
        EDIS;
        // Light up the fault LED
        GpioDataRegs.GPBCLEAR.bit.GPIO34 = 1;
    }
    else
    {
        EALLOW;
        EPwm1Regs.TZCLR.bit.OST = 1;
        EPwm2Regs.TZCLR.bit.OST = 1;
        EDIS;
        // Light OFF the fault LED
        GpioDataRegs.GPBSET.bit.GPIO34 = 1;
    }

    //___________
    //|      |
    //Q1     Q3
    //|___U   |___V
    //|      |
    //Q2     Q4
    //|______|_____
    if(N_fsw < (SWperLineCyc_Setting >> 1)) //positive half cycle
    {
        if((abs(Ifb_V_real) >= OCP_LV_UP) || (abs(Ifb_U_real) >= OCP_LV_UP))  //125
        {
            #if OCP_ACT == 0
            //PWM1A: Q1, ~Q2
            EPwm1Regs.CMPA.half.CMPA = 0;
            EPwm1Regs.CMPB = TBPRD_Setting;//0;
            //PWM2A: Q3, ~Q4
            EPwm2Regs.CMPA.half.CMPA = 0;
            EPwm2Regs.CMPB = TBPRD_Setting; //0;
            #else
            //turn off all devices
            //PWM1A: Q1, ~Q2
            EPwm1Regs.CMPA.half.CMPA = 0;
            EPwm1Regs.CMPB = 0;
            //PWM2A: Q3, ~Q4
            EPwm2Regs.CMPA.half.CMPA = 0;
            EPwm2Regs.CMPB = 0;
            #endif
        }
        else
        {
            //get theta in Q format
            theta_iq = ((long)N_fsw * Cnt_A) << (GLOBAL_Q - Shift_B - Shift_C);//PU //201>>5 = 2* PI
            //theta_iq = _IQ(N_fsw*2*PI) >> (SWperLineCyc_Setting_N);
            sintheta_iq = _IQsinPU(theta_iq);
            IQCMPRA = _IQmpy(sintheta_iq,mod_u_iq);
            IQCMPRA = (int)((((long)IQCMPRA >> 7) * TBPRD_Setting) >> 17);  //TBPRD_Setting
            //PWM1A: Q1, ~Q2
            EPwm1Regs.CMPA.half.CMPA = IQCMPRA;
            EPwm1Regs.CMPB = TBPRD_Setting - IQCMPRA;
            //PWM2A: Q3, ~Q4
            EPwm2Regs.CMPA.half.CMPA = 0;
            EPwm2Regs.CMPB = TBPRD_Setting;
        }
    }
    else
    {
        if((abs(Ifb_V_real) >= OCP_LV_UP) || (abs(Ifb_U_real) >= OCP_LV_UP))  //300
        {
            #if OCP_ACT == 0
            // only turn off active devices
            //PWM1A: Q1, ~Q2
            EPwm1Regs.CMPA.half.CMPA = 0;
            EPwm1Regs.CMPB = TBPRD_Setting;//0;
            //PWM2A: Q3, ~Q4
            EPwm2Regs.CMPA.half.CMPA = 0;
            EPwm2Regs.CMPB = TBPRD_Setting;
            #else
            // turn off all devices
            //PWM1A: Q1, ~Q2
            EPwm1Regs.CMPA.half.CMPA = 0;
            EPwm1Regs.CMPB = 0;
            //PWM2A: Q3, ~Q4
            EPwm2Regs.CMPA.half.CMPA = 0;
            EPwm2Regs.CMPB = 0;
            #endif
        }
        else
        {
            //get theta in Q format
            theta_iq = ((long)N_fsw * Cnt_A) << (GLOBAL_Q - Shift_B - Shift_C);//PU //201>>5 = 2* PI
            //theta_iq = _IQ(N_fsw*2*PI) >> (SWperLineCyc_Setting_N);
            sintheta_iq = _IQsinPU(theta_iq);
            IQCMPRA = _IQmpy(sintheta_iq,mod_u_iq);
            IQCMPRA = (int)((((long)IQCMPRA >> 7) * TBPRD_Setting) >> 17);  //TBPRD_Setting shift by 6 bit to avoid overflow
            //PWM1A: Q1, ~Q2
            EPwm1Regs.CMPA.half.CMPA = 0;
            EPwm1Regs.CMPB = TBPRD_Setting;
            //PWM2A: Q3, ~Q4
            //sintheta_temp = _IQ16sin(theta_temp);
            EPwm2Regs.CMPA.half.CMPA = 0 - IQCMPRA;
            EPwm2Regs.CMPB = TBPRD_Setting + IQCMPRA;
        }
    }

	// Clear INT flag for this timer
	EPwm1Regs.ETCLR.bit.INT = 1;

	// Acknowledge this interrupt to receive more interrupts from group 3
	PieCtrlRegs.PIEACK.all = PIEACK_GROUP3;
}

interrupt void epwm3_isr(void)
{
	EINT;

	EPwm3TimerIntCount++;

	if(EPwm3TimerIntCount >= 5000)
	{
		EPwm3TimerIntCount = 1;
		GpioDataRegs.GPATOGGLE.bit.GPIO31 = 1;
	}

    // Record how many points are collected during a half-cycle.
    #if FLINE_SETTING == 60 //60Hz
	    MainPoint = 167;
    #else  //50Hz
	    MainPoint = 200;
    #endif

	// MainPoint ++;
    // Calculate the RMS value of input voltage and current;
    VSqrtSum += ((long)(Vac_real) * (Vac_real) >> 4);

    ISqrtSum_U += (long)(Ifb_U_real) * (Ifb_U_real);
    ISqrtSum_V += (long)(Ifb_V_real) * (Ifb_V_real);

    if(flag_ZC == 1)
    {
        #if FLINE_SETTING == 60
           if(SubCnt == 1)
           {
               // temp1 = _IQ1div(_IQ1(VSqrtSum), _IQ1(MainPoint));
               tempV1 = ((VSqrtSum * 49) >> 12) << 4; //=1/167=49/2^13 *2 to get IQ1 format
               // the product is now in Q1 format
               Vac_rms = (int)_IQ1sqrt(tempV1);
               // Q0 format
               Vac_rms = (Vac_rms >> 1); //Q0 right shift by 1 and left shift by 2

               // Digital filter on Vrms, which sampling frequency is 120Hz.
               // Vrms_f:  Vrms value filtered fc=2.74Hz.
               Vac_rms_f += ((Vac_rms - Vac_rms_f) >> 2);
               SubCnt ++; // =2 to calculate current I
               VSqrtSum = 0;
           }
           if(SubCnt == 2)
           {
               //Calculate current RMS value, phase U
               //tempI1 = _IQ1div(_IQ1(ISqrtSum_U), _IQ1(MainPoint));
               tempI1 = (ISqrtSum_U * 49) >> 12;
               // the product is now in Q1 format
               Ifb_U_rms = _IQ1sqrt(tempI1);
               // Q0 format
               Ifb_U_rms = (Ifb_U_rms >> 1); //Q0 right shift by 1 and left shift by 2

               // Digital filter on Vrms, which sampling frequency is 120Hz.
               // Vrms_f:  Vrms value filtered fc=2.74Hz.
               Ifb_U_rms_f += ((Ifb_U_rms - Ifb_U_rms_f) >> 2);
               SubCnt ++; //=3 to calculate current 2
               ISqrtSum_U = 0;
           }
           if(SubCnt == 3)
           {
               //Calculate current RMS value, phase V
               //tempI2 = _IQ1div(_IQ1(ISqrtSum_V), _IQ1(MainPoint));
               tempI2 = (ISqrtSum_V * 49) >> 12;
               // the product is now in Q1 format
               Ifb_V_rms = _IQ1sqrt(tempI2);
               // Q0 format
               Ifb_V_rms = (Ifb_V_rms >> 1); //Q0 right shift by 1 and left shift by 2

               // Digital filter on Vrms, which sampling frequency is 120Hz.
               // Vrms_f:  Vrms value filtered fc=2.74Hz.
               Ifb_V_rms_f += ((Ifb_V_rms - Ifb_V_rms_f) >> 2);
               SubCnt = 0;  //reset
               ISqrtSum_V = 0;
               //MainPoint = 0;
               if ( ((Ifb_V_rms_f + Ifb_U_rms) >>1 ) >= If_rms_OCP)   //0.72*OCP_PK
               {
                  RMS_OCP_Cnt ++;  //counting time
                  if(RMS_OCP_Cnt >= OCP_TIME_OUT)
                  {
                      RMS_OCP_Flag = 1; //Set RMS OCP Flag to turn off PWM
                  }
               }
               else
               {
                   if(RMS_OCP_Cnt > 0)
                   {
                       RMS_OCP_Cnt --;
                   }
                   else
                   {
                       RMS_OCP_Cnt = 0;
                   }
               }

               flag_ZC = 0; //Reset zero crossing flag
           }
        #else //50Hz
            if(SubCnt == 1)
            {
                // temp1 = _IQ1div(_IQ1(VSqrtSum), _IQ1(MainPoint));
                tempV1 = ((VSqrtSum * 41) >> 12) << 4; //=1/200=41/2^13 *2 to get IQ1 format
                // the product is now in Q1 format
                Vac_rms = (int)_IQ1sqrt(tempV1);
                // Q0 format
                Vac_rms = (Vac_rms >> 1); //Q0 right shift by 1 and left shift by 2

                // Digital filter on Vrms, which sampling frequency is 120Hz.
                // Vrms_f:  Vrms value filtered fc=2.74Hz.
                Vac_rms_f += ((Vac_rms - Vac_rms_f) >> 2);
                SubCnt ++; // =2 to calculate current I
            }
            if(SubCnt == 2)
            {
                //Calculate current RMS value, phase U
                //tempI1 = _IQ1div(_IQ1(ISqrtSum_U), _IQ1(MainPoint));
                tempI1 = (ISqrtSum_U * 41) >> 12;
                // the product is now in Q1 format
                Ifb_U_rms = _IQ1sqrt(tempI1);
                // Q0 format
                Ifb_U_rms = (Ifb_U_rms >> 1); //Q0 right shift by 1 and left shift by 2

                // Digital filter on Vrms, which sampling frequency is 120Hz.
                // Vrms_f:  Vrms value filtered fc=2.74Hz.
                Ifb_U_rms_f += ((Ifb_U_rms - Ifb_U_rms_f) >> 2);
                SubCnt ++; //=3 to calculate current 2
            }
            if(SubCnt == 3)
            {
                //Calculate current RMS value, phase V
                //tempI2 = _IQ1div(_IQ1(ISqrtSum_V), _IQ1(MainPoint));
                tempI2 = (ISqrtSum_V * 41) >> 12;
                // the product is now in Q1 format
                Ifb_V_rms = _IQ1sqrt(tempI2);
                // Q0 format
                Ifb_V_rms = (Ifb_V_rms >> 1); //Q0 right shift by 1 and left shift by 2

                // Digital filter on Vrms, which sampling frequency is 120Hz.
                // Vrms_f:  Vrms value filtered fc=2.74Hz.
                Ifb_V_rms_f += ((Ifb_V_rms - Ifb_V_rms_f) >> 2);
                SubCnt = 0;  //reset
                VSqrtSum = 0;
                ISqrtSum_U = 0;
                ISqrtSum_V = 0;
                //MainPoint = 0;

                if ( ((Ifb_V_rms_f + Ifb_U_rms) >>1 ) >= If_rms_OCP)   //0.72*OCP_PK
                {
                   RMS_OCP_Cnt ++;  //counting time
                   if(RMS_OCP_Cnt >= OCP_TIME_OUT)
                   {
                       RMS_OCP_Flag = 1; //Set RMS OCP Flag to turn off PWM
                   }
                }
                else
                {
                    if(RMS_OCP_Cnt > 0)
                    {
                        RMS_OCP_Cnt --;
                    }
                    else
                    {
                        RMS_OCP_Cnt = 0;
                    }
                }
                flag_ZC = 0; //Reset zero crossing flag
            }
        #endif
    }
	// Clear INT flag for this timer
	EPwm3Regs.ETCLR.bit.INT = 1;

	// Acknowledge this interrupt to receive more interrupts from group 3
	PieCtrlRegs.PIEACK.all = PIEACK_GROUP3;
}

void InitEPwm1Gpio(void)
{
   EALLOW;

/* Disable internal pull-up for the selected output pins
   for reduced power consumption */
// Pull-ups can be enabled or disabled by the user.
// Comment out other unwanted lines.

    GpioCtrlRegs.GPAPUD.bit.GPIO0 = 1;    // Disable pull-up on GPIO0 (EPWM1A)
    GpioCtrlRegs.GPAPUD.bit.GPIO1 = 1;    // Disable pull-up on GPIO1 (EPWM1B)

/* Configure EPWM-1 pins using GPIO regs*/
// This specifies which of the possible GPIO pins will be EPWM1 functional pins.
// Comment out other unwanted lines.

    GpioCtrlRegs.GPAMUX1.bit.GPIO0 = 1;   // Configure GPIO0 as EPWM1A
    GpioCtrlRegs.GPAMUX1.bit.GPIO1 = 1;   // Configure GPIO1 as EPWM1B

    EDIS;
}

void InitEPwm2Gpio(void)
{
	EALLOW;
	GpioCtrlRegs.GPAPUD.bit.GPIO2 = 1;    // Disable pull-up on GPIO2 (EPWM2A)
	GpioCtrlRegs.GPAPUD.bit.GPIO3 = 1;    // Disable pull-up on GPIO3 (EPWM2B)
	GpioCtrlRegs.GPAMUX1.bit.GPIO2 = 1;   // Configure GPIO2 as EPWM2A
	GpioCtrlRegs.GPAMUX1.bit.GPIO3 = 1;   // Configure GPIO3 as EPWM2B
	EDIS;
}

void InitEPwm3Gpio(void)
{
   EALLOW;

/* Disable internal pull-up for the selected output pins
   for reduced power consumption */
// Pull-ups can be enabled or disabled by the user.
// This will enable the pullups for the specified pins.
// Comment out other unwanted lines.

    GpioCtrlRegs.GPAPUD.bit.GPIO4 = 1;    // Disable pull-up on GPIO4 (EPWM3A)
    GpioCtrlRegs.GPAPUD.bit.GPIO5 = 1;    // Disable pull-up on GPIO5 (EPWM3B)

/* Configure EPwm-3 pins using GPIO regs*/
// This specifies which of the possible GPIO pins will be EPWM3 functional pins.
// Comment out other unwanted lines.

    GpioCtrlRegs.GPAMUX1.bit.GPIO4 = 1;   // Configure GPIO4 as EPWM3A
    GpioCtrlRegs.GPAMUX1.bit.GPIO5 = 1;   // Configure GPIO5 as EPWM3B

    EDIS;
}


void InitEPwm1Example()
{
	EPwm1Regs.TBPRD = TBPRD_Setting;               	// Set timer Period 50KHz or 100KHz
	EPwm1Regs.TBPHS.half.TBPHS = 0x0000;           	// Set Phase register to 0
	EPwm1Regs.TBCTR = 0x0000;                      	// Clear counter

	// Setup TBCLK
	EPwm1Regs.TBCTL.bit.CTRMODE = TB_COUNT_UPDOWN; 	// Count up and down
	EPwm1Regs.TBCTL.bit.PHSEN = TB_DISABLE;        	// Disable phase loading
	EPwm1Regs.TBCTL.bit.PRDLD = TB_SHADOW;
	EPwm1Regs.TBCTL.bit.SYNCOSEL = TB_CTR_ZERO;    	// Sync down-stream module

	EPwm1Regs.TBCTL.bit.HSPCLKDIV = TB_DIV1;       	// Clock ratio to SYSCLKOUT
	EPwm1Regs.TBCTL.bit.CLKDIV = TB_DIV1;

	EPwm1Regs.CMPCTL.bit.SHDWAMODE = CC_SHADOW;    	// Load registers in shadow mode
	EPwm1Regs.CMPCTL.bit.SHDWBMODE = CC_SHADOW;
	EPwm1Regs.CMPCTL.bit.LOADAMODE = CC_CTR_ZERO;   // no effect in immediately mode
	EPwm1Regs.CMPCTL.bit.LOADBMODE = CC_CTR_ZERO;

	EPwm1Regs.CMPA.half.CMPA = 0;               	// Setup initial compare value
	EPwm1Regs.CMPB = 0;

	// Set actions
	EPwm1Regs.AQCTLA.bit.CAU = AQ_CLEAR;     		// Clear PWM1A when CMPA equals the counter and the counter is incrementing.
	EPwm1Regs.AQCTLA.bit.CAD = AQ_SET;       		// Set PWM1A when CMPA equals the counter and the counter is decrementing.
	EPwm1Regs.AQCTLB.bit.CBU = AQ_CLEAR;       		// Clear PWM1B when CMPB equals the counter and the counter is incrementing.
	EPwm1Regs.AQCTLB.bit.CBD = AQ_SET;       		// Set PWM1B when CMPB equals the counter and the counter is decrementing.

	// Set Dead Time
	EPwm1Regs.DBCTL.bit.IN_MODE = DBA_ALL;			// Enable Dead-band mode, Classical dead-band mode
	EPwm1Regs.DBCTL.bit.OUT_MODE = DB_FULL_ENABLE;	// Enable Dead-band mode
	EPwm1Regs.DBCTL.bit.POLSEL = DB_ACTV_HIC;		// Active Hi Complementary
	EPwm1Regs.DBFED = 0;							// FED = 15 TBCLKs use Si8230's dead time setting
	EPwm1Regs.DBRED = 0;							// RED = 15 TBCLKs

	//Trip event setup
    //Trip event setup
    EALLOW;
    EPwm1Regs.TZSEL.bit.OSHT1 = 1;          // enable one-shot trip for PWM1
    EPwm1Regs.TZCTL.bit.TZA = TZ_FORCE_LO;  // set EPWM1A to low at default
    EPwm1Regs.TZFRC.bit.OST = 1;            // shutdown EPWM1 at beginning

	//EPwm1Regs.TZSEL.bit.OSHT1 = 0;  				// Disable one-shot trip for PWM1
	//EPwm1Regs.TZCTL.bit.TZA = TZ_NO_CHANGE;   		// set EPWM1A to low at default
	EDIS;

	// Interrupt where we have the control
	EPwm1Regs.ETSEL.bit.INTEN = 1;              	// Enable INT
	EPwm1Regs.ETSEL.bit.INTSEL = ET_CTR_PRD; 		// INT on counter-zero
	EPwm1Regs.ETPS.bit.INTPRD = ET_1ST;        		// Generate pulse on 1st event

	EPwm1Regs.ETSEL.bit.SOCAEN	= 1;				// Enable SOC on A group
	EPwm1Regs.ETSEL.bit.SOCASEL	= ET_CTR_PRD;		// Select SOC on counter-zero event
	EPwm1Regs.ETPS.bit.SOCAPRD 	= ET_1ST;			// Generate pulse on 1st event
}


void InitEPwm2Example()
{
	EPwm2Regs.TBPRD = TBPRD_Setting;                // Set timer period Period, 50KHz or 100KHz
	EPwm2Regs.TBPHS.half.TBPHS = 0x0000;           	// Phase is 0
	EPwm2Regs.TBCTR = 0x0000;                      	// Clear counter

	// Setup TBCLK
	EPwm2Regs.TBCTL.bit.CTRMODE = TB_COUNT_UPDOWN; 	// Count up and down
	EPwm2Regs.TBCTL.bit.PHSEN = TB_ENABLE;        	// Enable phase loading
	EPwm2Regs.TBCTL.bit.PRDLD = TB_SHADOW;

	EPwm2Regs.TBCTL.bit.SYNCOSEL = TB_SYNC_IN; 		//sync flow-through

	EPwm2Regs.CMPCTL.bit.SHDWAMODE = CC_SHADOW;    	// Load registers in the shadow mode
	EPwm2Regs.CMPCTL.bit.SHDWBMODE = CC_SHADOW;
	EPwm2Regs.CMPCTL.bit.LOADAMODE = CC_CTR_ZERO;   // no effect in immediately mode
	EPwm2Regs.CMPCTL.bit.LOADBMODE = CC_CTR_ZERO;

	EPwm2Regs.TBCTL.bit.HSPCLKDIV = TB_DIV1;       	// Clock ratio to SYSCLKOUT
	EPwm2Regs.TBCTL.bit.CLKDIV = TB_DIV1;          	// Slow just to observe on the scope

	// Setup the initial compare value
	EPwm2Regs.CMPA.half.CMPA = 0;
	EPwm2Regs.CMPB = 0;

	// Set actions
	EPwm2Regs.AQCTLA.bit.CAU = AQ_CLEAR;    	// Clear PWM2A when CMPA equals the counter and the counter is incrementing.
	EPwm2Regs.AQCTLA.bit.CAD = AQ_SET;   	// Set PWM2A when CMPA equals the counter and the counter is decrementing.
	EPwm2Regs.AQCTLB.bit.CBU = AQ_CLEAR;//AQ_SET;   	// Set PWM2B when CMPB equals the counter and the counter is decrementing.
	EPwm2Regs.AQCTLB.bit.CBD = AQ_SET;//AQ_CLEAR;   	// Clear PWM2B when CMPB equals the counter and the counter is incrementing.

	// Set Dead Time
	EPwm2Regs.DBCTL.bit.IN_MODE = DBA_ALL;			// Enable Dead-band mode, Classical dead-band mode
	EPwm2Regs.DBCTL.bit.OUT_MODE = DB_FULL_ENABLE;	// Enable Dead-band mode
	EPwm2Regs.DBCTL.bit.POLSEL = DB_ACTV_HIC;		// Active Hi Complementary
	EPwm2Regs.DBFED = 0;							// FED = 12 TBCLKs use Si8230's dead time setting
	EPwm2Regs.DBRED = 0;							// RED = 12 TBCLKs

	//Trip event set up
	EALLOW;
    EPwm2Regs.TZSEL.bit.OSHT1 = 1;              //enable one-shot trip for PWM1
    EPwm2Regs.TZCTL.bit.TZA = TZ_FORCE_LO;      // set EPWM2A to high at default
    EPwm2Regs.TZFRC.bit.OST = 1;                // shutdown EPWM2 at the beginning

	//EPwm2Regs.TZSEL.bit.OSHT1 = 0;  			// Disable one-shot trip for PWM1
	//EPwm2Regs.TZCTL.bit.TZA = TZ_NO_CHANGE;   	// Set EPWM2A to high at default
	EDIS;

	// Interrupt where we have the control
	EPwm2Regs.ETSEL.bit.INTEN = 1;              	// Enable INT
	EPwm2Regs.ETSEL.bit.INTSEL = ET_CTR_ZERO; 		// INT on counter-period
	EPwm2Regs.ETPS.bit.INTPRD = ET_1ST;        		// Generate pulse on 1st event

	EPwm2Regs.ETSEL.bit.SOCAEN	= 1;				// Enable SOC on A group
	EPwm2Regs.ETSEL.bit.SOCASEL	= ET_CTR_ZERO;		// Select SOC on counter-zero event
	EPwm2Regs.ETPS.bit.SOCAPRD 	= ET_1ST;			// Generate pulse on 1st event
}

void InitEPwm3Example()
{
	EPwm3Regs.TBPRD = TBPRD_10KHz;                 // Set timer period 60M/3000/2 = 10KhZ
	EPwm3Regs.TBPHS.half.TBPHS = 0x0000;            // Phase is 0
	EPwm3Regs.TBCTR = 0x0000;                       // Clear counter

	// Setup TBCLK
	EPwm3Regs.TBCTL.bit.CTRMODE = TB_COUNT_UPDOWN; // Count up
	EPwm3Regs.TBCTL.bit.PHSEN = TB_DISABLE;        // Disable phase loading
	EPwm3Regs.TBCTL.bit.HSPCLKDIV = TB_DIV1;       // Clock ratio to SYSCLKOUT
	EPwm3Regs.TBCTL.bit.CLKDIV = TB_DIV1;          // Slow so we can observe on the scope
	EPwm3Regs.TBCTL.bit.SYNCOSEL = TB_CTR_ZERO;

	// slow Interrupt at 10KHz
	EPwm3Regs.ETSEL.bit.INTEN = 1;               //
	EPwm3Regs.ETSEL.bit.INTSEL = ET_CTR_ZERO;   // Select Int from counter = PRD
	EPwm3Regs.ETPS.bit.INTPRD = ET_1ST;        // Generate pulse on 1st event

	EPwm3Regs.AQSFRC.bit.RLDCSF = 0x3;      // load immediately

}

//---------------------------------------------------------------------------
// InitGPIOs:
//---------------------------------------------------------------------------
// This function initializes the Gpio to a known (default) state.
//
// For more details on configuring GPIO's as peripheral functions,
// refer to the individual peripheral examples and/or GPIO setup example.
void InitGPIOs(void)
{
   EALLOW;

	GpioCtrlRegs.GPAPUD.bit.GPIO16 = 0;   // Enable pullup on GPIO16
	GpioCtrlRegs.GPAMUX2.bit.GPIO16 = 0;  // GPIO16 = GPIO
	GpioCtrlRegs.GPADIR.bit.GPIO16 = 0;   // GPIO16 = input

	GpioCtrlRegs.GPAPUD.bit.GPIO17 = 0;   // Enable pullup on GPIO17
	GpioCtrlRegs.GPAMUX2.bit.GPIO17 = 0;  // GPIO17 = GPIO
	GpioCtrlRegs.GPADIR.bit.GPIO17 = 1;   // GPIO17 = output
    GpioDataRegs.GPACLEAR.bit.GPIO17 = 0;   //set output Low

	GpioCtrlRegs.GPAPUD.bit.GPIO18 = 0;   // Enable pullup on GPIO18
	GpioCtrlRegs.GPAMUX2.bit.GPIO18 = 0;  // GPIO18 = GPIO
	GpioCtrlRegs.GPADIR.bit.GPIO18 = 0;   // GPIO18 = input

	GpioCtrlRegs.GPAPUD.bit.GPIO19 = 0;   // Enable pullup on GPIO19
	GpioCtrlRegs.GPAMUX2.bit.GPIO19 = 0;  // GPIO19 = GPIO
	GpioCtrlRegs.GPADIR.bit.GPIO19 = 0;   // GPIO19 = input

	GpioCtrlRegs.GPAPUD.bit.GPIO20 = 0;   // Enable pullup on GPIO20
	GpioCtrlRegs.GPAMUX2.bit.GPIO20 = 0;  // GPIO20 = GPIO
	GpioCtrlRegs.GPADIR.bit.GPIO20 = 0;   // GPIO20 = input

	GpioCtrlRegs.GPAPUD.bit.GPIO21 = 0;   // Enable pullup on GPIO21
	GpioCtrlRegs.GPAMUX2.bit.GPIO21 = 0;  // GPIO21 = GPIO
	GpioCtrlRegs.GPADIR.bit.GPIO21 = 0;   // GPIO21 = input

	GpioCtrlRegs.GPAPUD.bit.GPIO22 = 0;   // Enable pullup on GPIO22
	GpioCtrlRegs.GPAMUX2.bit.GPIO22 = 0;  // GPIO22 = GPIO
	GpioCtrlRegs.GPADIR.bit.GPIO22 = 0;   // GPIO22 = input

	GpioCtrlRegs.GPAPUD.bit.GPIO23 = 0;   // Enable pullup on GPIO23
	GpioCtrlRegs.GPAMUX2.bit.GPIO23 = 0;  // GPIO23 = GPIO
	GpioCtrlRegs.GPADIR.bit.GPIO23 = 0;   // GPIO23 = input

 	GpioCtrlRegs.GPAPUD.bit.GPIO31 = 0;   // Enable pullup on GPIO12
	GpioCtrlRegs.GPAMUX2.bit.GPIO31 = 0;  // GPIO12 = GPIO12
	GpioCtrlRegs.GPADIR.bit.GPIO31 = 1;   //GPIO as output
	GpioDataRegs.GPACLEAR.bit.GPIO31 = 0;   //set output Low

	GpioCtrlRegs.GPBPUD.bit.GPIO34 = 0;   // Enable pullup on GPIO18
	GpioCtrlRegs.GPBMUX1.bit.GPIO34 = 0;  // GPIO18 = GPIO18
	GpioCtrlRegs.GPBDIR.bit.GPIO34 = 1;   //GPIO as output
	GpioDataRegs.GPBSET.bit.GPIO34 = 1;   //set output Low
	GpioDataRegs.GPBCLEAR.bit.GPIO34 = 0;   //set output HIGH
   EDIS;

}


void ConfigAdc1()  // configure ADC, first sample the Vref
{
    EALLOW;

    //configure ADC,
    AdcRegs.ADCSOC0CTL.bit.CHSEL    = 0;    //set SOC0 channel select to ADCINA0
    AdcRegs.ADCSOC1CTL.bit.CHSEL    = 0;    //set SOC1 channel select to ADCINA0
    AdcRegs.ADCSOC2CTL.bit.CHSEL    = 0;    //set SOC2 channel select to ADCINA0
    AdcRegs.ADCSOC3CTL.bit.CHSEL    = 0;    //set SOC3 channel select to ADCINA0
    AdcRegs.ADCSOC4CTL.bit.CHSEL    = 0;    //set SOC4 channel select to ADCINA0
    AdcRegs.ADCSOC5CTL.bit.CHSEL    = 4;    //set SOC0 channel select to ADCINA4
    AdcRegs.ADCSOC6CTL.bit.CHSEL    = 4;    //set SOC1 channel select to ADCINA4
    AdcRegs.ADCSOC7CTL.bit.CHSEL    = 4;    //set SOC2 channel select to ADCINA4
    AdcRegs.ADCSOC8CTL.bit.CHSEL    = 4;    //set SOC3 channel select to ADCINA4
    AdcRegs.ADCSOC9CTL.bit.CHSEL    = 12;    //set SOC4 channel select to ADCINA12
    AdcRegs.ADCSOC10CTL.bit.CHSEL    = 12;    //set SOC0 channel select to ADCINA12
    AdcRegs.ADCSOC11CTL.bit.CHSEL    = 12;    //set SOC1 channel select to ADCINA12
    AdcRegs.ADCSOC12CTL.bit.CHSEL    = 12;    //set SOC2 channel select to ADCINA12


    AdcRegs.ADCSOC0CTL.bit.TRIGSEL  = 0;    //set SOC0 start trigger on software, due to round-robin SOC0 converts first then SOC1
    AdcRegs.ADCSOC1CTL.bit.TRIGSEL  = 0;    //set SOC1 start trigger on software, due to round-robin SOC0 converts first then SOC1
    AdcRegs.ADCSOC2CTL.bit.TRIGSEL  = 0;    //set SOC2 start trigger on software, due to round-robin SOC0 converts first then SOC1, then SOC2
    AdcRegs.ADCSOC3CTL.bit.TRIGSEL  = 0;
    AdcRegs.ADCSOC4CTL.bit.TRIGSEL  = 0;
    AdcRegs.ADCSOC5CTL.bit.TRIGSEL  = 0;    //set SOC0 start trigger on software, due to round-robin SOC0 converts first then SOC1
    AdcRegs.ADCSOC6CTL.bit.TRIGSEL  = 0;    //set SOC1 start trigger on software, due to round-robin SOC0 converts first then SOC1
    AdcRegs.ADCSOC7CTL.bit.TRIGSEL  = 0;    //set SOC2 start trigger on software, due to round-robin SOC0 converts first then SOC1, then SOC2
    AdcRegs.ADCSOC8CTL.bit.TRIGSEL  = 0;
    AdcRegs.ADCSOC9CTL.bit.TRIGSEL  = 0;
    AdcRegs.ADCSOC10CTL.bit.TRIGSEL  = 0;    //set SOC2 start trigger on software, due to round-robin SOC0 converts first then SOC1, then SOC2
    AdcRegs.ADCSOC11CTL.bit.TRIGSEL  = 0;
    AdcRegs.ADCSOC12CTL.bit.TRIGSEL  = 0;

    AdcRegs.ADCSOC0CTL.bit.ACQPS    = 6;    //set SOC0 S/H Window to 7 ADC Clock Cycles, (6 ACQPS plus 1)
    AdcRegs.ADCSOC1CTL.bit.ACQPS    = 6;    //set SOC1 S/H Window to 7 ADC Clock Cycles, (6 ACQPS plus 1)
    AdcRegs.ADCSOC2CTL.bit.ACQPS    = 6;    //set SOC2 S/H Window to 7 ADC Clock Cycles, (6 ACQPS plus 1)
    AdcRegs.ADCSOC3CTL.bit.ACQPS    = 6;
    AdcRegs.ADCSOC4CTL.bit.ACQPS    = 6;
    AdcRegs.ADCSOC5CTL.bit.ACQPS    = 6;    //set SOC0 S/H Window to 7 ADC Clock Cycles, (6 ACQPS plus 1)
    AdcRegs.ADCSOC6CTL.bit.ACQPS    = 6;    //set SOC1 S/H Window to 7 ADC Clock Cycles, (6 ACQPS plus 1)
    AdcRegs.ADCSOC7CTL.bit.ACQPS    = 6;    //set SOC2 S/H Window to 7 ADC Clock Cycles, (6 ACQPS plus 1)
    AdcRegs.ADCSOC8CTL.bit.ACQPS    = 6;
    AdcRegs.ADCSOC9CTL.bit.ACQPS    = 6;
    AdcRegs.ADCSOC10CTL.bit.ACQPS    = 6;    //set SOC2 S/H Window to 7 ADC Clock Cycles, (6 ACQPS plus 1)
    AdcRegs.ADCSOC11CTL.bit.ACQPS    = 6;
    AdcRegs.ADCSOC12CTL.bit.ACQPS    = 6;

    EDIS;

}



void ConfigAdc2()  // configure ADC to sample current and voltage, for the control purpose
{
    EALLOW;

    //configure ADC,034 12
    AdcRegs.ADCSOC0CTL.bit.CHSEL    = 0;        //set SOC0 channel select to ADCINA0 ---- Vac
    AdcRegs.ADCSOC1CTL.bit.CHSEL    = 0;        //set SOC1 channel select to ADCINA0 ---- Vac
    AdcRegs.ADCSOC2CTL.bit.CHSEL    = 3;        //set SOC2 channel select to ADCINA3 ---- Vdc
    AdcRegs.ADCSOC3CTL.bit.CHSEL    = 4;        //set SOC3 channel select to ADCINA4 ---- Ifb_V
    AdcRegs.ADCSOC4CTL.bit.CHSEL    = 12;       //set SOC4 channel select to ADCINB4 ---- Ifb_U

    AdcRegs.ADCSOC0CTL.bit.TRIGSEL  = 5;    //set SOC0 start trigger on EPWM1A, due to round-robin SOC0 converts first then SOC1
    AdcRegs.ADCSOC1CTL.bit.TRIGSEL  = 5;
    AdcRegs.ADCSOC2CTL.bit.TRIGSEL  = 5;    //set SOC1 start trigger on EPWM1A, due to round-robin SOC0 converts first then SOC1
    AdcRegs.ADCSOC3CTL.bit.TRIGSEL  = 5;    //set SOC2 start trigger on EPWM1A, due to round-robin SOC0 converts first then SOC1, then SOC2
    AdcRegs.ADCSOC4CTL.bit.TRIGSEL  = 5;

    AdcRegs.ADCSOC0CTL.bit.ACQPS    = 6;
    AdcRegs.ADCSOC1CTL.bit.ACQPS    = 6;    //set SOC0 S/H Window to 7 ADC Clock Cycles, (6 ACQPS plus 1)
    AdcRegs.ADCSOC2CTL.bit.ACQPS    = 6;    //set SOC1 S/H Window to 7 ADC Clock Cycles, (6 ACQPS plus 1)
    AdcRegs.ADCSOC3CTL.bit.ACQPS    = 6;    //set SOC2 S/H Window to 7 ADC Clock Cycles, (6 ACQPS plus 1)
    AdcRegs.ADCSOC4CTL.bit.ACQPS    = 6;

    EDIS;

}

int GetPWM_Mode(void)
{
	//GPIO20,21,22,23
	//    0
	if(GpioDataRegs.GPADAT.bit.GPIO20 == 0)
	{
		if(GpioDataRegs.GPADAT.bit.GPIO21 == 0)
		{
			return 0;  //default mode, semi unipolar
		}
		else
		{
			return 1;  //unipolar
		}
	}
	else
	{
		if(GpioDataRegs.GPADAT.bit.GPIO21 == 0)
		{
			return 2;  //bipolar PWM
		}
		else
		{
			return 3;  //totem pole
		}
	}
}



//No More


